"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var gulp_core_build_1 = require('@microsoft/gulp-core-build');
var ts = require('gulp-typescript');
var path = require('path');
var TypeScriptTask = (function (_super) {
    __extends(TypeScriptTask, _super);
    function TypeScriptTask() {
        var _this = this;
        _super.apply(this, arguments);
        this.name = 'typescript';
        this.taskConfig = {
            failBuildOnErrors: true,
            reporter: {
                error: function (error) {
                    var filename = error.relativeFilename || error.fullFilename;
                    var line = error.startPosition ? error.startPosition.line : 0;
                    var character = error.startPosition ? error.startPosition.character : 0;
                    var code = error.diagnostic.code;
                    var errorMessage = (typeof error.diagnostic.messageText === 'object') ?
                        error.diagnostic.messageText.messageText :
                        error.diagnostic.messageText;
                    _this.fileError(filename, line, character, 'TS' + code, errorMessage);
                }
            },
            sourceMatch: [
                'src/**/*.ts',
                'src/**/*.tsx',
                'typings/main/**/*.ts',
                'typings/main.d.ts',
                'typings/tsd.d.ts',
                'typings/index.d.ts'
            ],
            staticMatch: [
                'src/**/*.js',
                'src/**/*.json',
                'src/**/*.jsx'
            ],
            removeCommentsFromJavaScript: false,
            emitSourceMaps: true
        };
    }
    TypeScriptTask.prototype.executeTask = function (gulp, completeCallback) {
        var _this = this;
        /* tslint:disable:typedef */
        var assign = require('object-assign');
        var merge = require('merge2');
        /* tslint:enable:typedef */
        var allStreams = [];
        var result = {
            errorCount: 0
        };
        /* tslint:disable:no-any */
        var tsConfig = this.readJSONSync('tsconfig.json');
        /* tslint:enable:no-any */
        // Set default config if no local tsconfig.json exists.
        if (!tsConfig) {
            tsConfig = {
                compilerOptions: {
                    'declaration': true,
                    'experimentalDecorators': true,
                    'jsx': 'react',
                    'moduleResolution': 'node',
                    'sourceMap': true,
                    'target': 'es5'
                }
            };
        }
        // Log the compiler version for custom verisons.
        if (this.taskConfig.typescript && this.taskConfig.typescript.version) {
            this.log("Using custom version: " + this.taskConfig.typescript.version);
        }
        var tsCompilerOptions = assign({}, tsConfig.compilerOptions, {
            module: 'commonjs',
            typescript: this.taskConfig.typescript
        });
        this._tsProject = this._tsProject || ts.createProject(tsCompilerOptions);
        this._compileProject(gulp, this._tsProject, this.buildConfig.libFolder, allStreams, result);
        // Static passthrough files.
        var staticSrc = gulp.src(this.taskConfig.staticMatch);
        allStreams.push(staticSrc.pipe(gulp.dest(this.buildConfig.libFolder)));
        // If AMD modules are required, also build that.
        if (this.buildConfig.libAMDFolder) {
            allStreams.push(staticSrc.pipe(gulp.dest(this.buildConfig.libAMDFolder)));
            this._tsAMDProject = this._tsAMDProject || ts.createProject(assign({}, tsCompilerOptions, { module: 'amd' }));
            this._compileProject(gulp, this._tsAMDProject, this.buildConfig.libAMDFolder, allStreams, result);
        }
        // Listen for pass/fail, and ensure that the task passes/fails appropriately.
        merge(allStreams)
            .on('queueDrain', function () {
            if (_this.taskConfig.failBuildOnErrors && result.errorCount) {
                completeCallback('TypeScript error(s) occurred.');
            }
            else {
                completeCallback();
            }
        })
            .on('error', completeCallback);
    };
    /** Override the new mergeConfig API */
    TypeScriptTask.prototype.mergeConfig = function (config) {
        throw 'Do not use mergeConfig with gulp-core-build-typescript';
    };
    TypeScriptTask.prototype._compileProject = function (gulp, tsProject, destDir, allStreams, result) {
        /* tslint:disable:typedef */
        var plumber = require('gulp-plumber');
        var sourcemaps = require('gulp-sourcemaps');
        /* tslint:enable:typedef */
        // tslint:disable-next-line:no-any
        var tsResult = gulp.src(this.taskConfig.sourceMatch)
            .pipe(plumber({
            errorHandler: function () {
                result.errorCount++;
            }
        }));
        if (this.taskConfig.emitSourceMaps) {
            tsResult = tsResult.pipe(sourcemaps.init());
        }
        tsResult = tsResult
            .pipe(ts(tsProject, undefined, this.taskConfig.reporter));
        // tslint:disable-next-line:typedef
        var jsResult = (this.taskConfig.removeCommentsFromJavaScript
            ? tsResult.js.pipe(require('gulp-decomment')({
                space: !!this.taskConfig.emitSourceMaps /* turn comments into spaces to preserve sourcemaps */
            }))
            : tsResult.js);
        if (this.taskConfig.emitSourceMaps) {
            jsResult = jsResult.pipe(sourcemaps.write('.', { sourceRoot: this._resolveSourceMapRoot }));
        }
        allStreams.push(jsResult
            .pipe(gulp.dest(destDir)));
        allStreams.push(tsResult.dts.pipe(gulp.dest(destDir)));
    };
    TypeScriptTask.prototype._resolveSourceMapRoot = function (file) {
        return path.relative(file.relative, path.join(file.cwd, 'src'));
    };
    return TypeScriptTask;
}(gulp_core_build_1.GulpTask));
exports.TypeScriptTask = TypeScriptTask;

//# sourceMappingURL=TypeScriptTask.js.map
