/// <reference types="es6-promise" />
/// <reference types="node" />
import { GulpProxy } from './GulpProxy';
import { IExecutable } from './IExecutable';
import { IBuildConfig } from './IBuildConfig';
export { IExecutable } from './IExecutable';
import * as gulp from 'gulp';
export * from './IBuildConfig';
export * from './logging';
export * from './tasks/CopyTask';
export * from './tasks/GenerateShrinkwrapTask';
export * from './tasks/GulpTask';
export * from './tasks/CleanTask';
export * from './tasks/ValidateShrinkwrapTask';
/**
 * Merges the given build config settings into existing settings.
 *
 * @param  {IBuildConfig} The build config settings.
 */
export declare function setConfig(config: IBuildConfig): void;
/**
 * Merges the given build config settings into existing settings.
 *
 * @param  {IBuildConfig} The build config settings.
 */
export declare function mergeConfig(config: IBuildConfig): void;
/**
 * Replaces the build config.
 *
 * @param  {IBuildConfig} config
 */
export declare function replaceConfig(config: IBuildConfig): void;
/**
 * Gets the current config.
 *
 * @returns IBuildConfig
 */
export declare function getConfig(): IBuildConfig;
/**
 * Defines a gulp task and maps it to a given IExecutable.
 *
 * @param  {string} taskName
 * @param  {IExecutable} task
 * @returns IExecutable
 */
export declare function task(taskName: string, task: IExecutable): IExecutable;
/**
 * The callback interface for a custom task definition.
 * The task should either return a Promise, a stream, or call the
 * callback function (passing in an object value if there was an error).
 */
export interface ICustomGulpTask {
    (gulp: gulp.Gulp | GulpProxy, buildConfig: IBuildConfig, done: (failure?: Object) => void): Promise<Object> | NodeJS.ReadWriteStream | void;
}
/**
 * Creates a new subtask from a function callback. Useful as a shorthand way
 * of defining tasks directly in a gulpfile.
 *
 * @param {string} taskName - the name of the task, appearing in build logs
 * @param {boolean} addCommandLine - true if this task should be registered to the command line
 * @param {ICustomGulpTask} fn - the callback function to execute when this task runs
 */
export declare function subTask(taskName: string, fn: ICustomGulpTask): IExecutable;
/**
 * Defines a gulp watch and maps it to a given IExecutable.
 *
 * @param  {string} watchMatch
 * @param  {IExecutable} task
 * @returns IExecutable
 */
export declare function watch(watchMatch: string | string[], task: IExecutable): IExecutable;
/**
 * Takes in IExecutables as arguments and returns an IExecutable that will execute them in serial.
 *
 * @param  {IExecutable[]} ...tasks
 * @returns IExecutable
 */
export declare function serial(...tasks: Array<IExecutable[] | IExecutable>): IExecutable;
/**
 * Takes in IExecutables as arguments and returns an IExecutable that will execute them in parallel.
 *
 * @param  {IExecutable[]} ...tasks
 * @returns IExecutable
 */
export declare function parallel(...tasks: Array<IExecutable[] | IExecutable>): IExecutable;
/**
 * Initializes the gulp tasks.
 *
 * @param  {any} gulp
 */
export declare function initialize(gulp: gulp.Gulp): void;
export declare const clean: IExecutable;
