"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var GulpTask_1 = require('./GulpTask');
var child_process = require('child_process');
var rimraf = require('rimraf');
var os = require('os');
var path = require('path');
/**
 * This provides a convenient way to more consistently generate a shrinkwrap file in
 * a desired manner as a gulp task, as there are many consistency issues with just
 * running npm-shrinkwrap directly.
 */
var GenerateShrinkwrapTask = (function (_super) {
    __extends(GenerateShrinkwrapTask, _super);
    function GenerateShrinkwrapTask() {
        _super.apply(this, arguments);
        this.name = 'generate-shrinkwrap';
    }
    GenerateShrinkwrapTask.prototype.executeTask = function (gulp) {
        var pathToShrinkwrap = path.join(this.buildConfig.rootPath, 'npm-shrinkwrap.json');
        if (this.fileExists(pathToShrinkwrap)) {
            this.log("Remove existing shrinkwrap file.");
            this._dangerouslyDeletePath(pathToShrinkwrap);
        }
        this.log("Running npm prune");
        child_process.execSync('npm prune');
        this.log("Running npm update");
        child_process.execSync('npm update');
        this.log("Running npm shrinkwrap --dev");
        child_process.execSync('npm shrinkwrap --dev');
        return;
    };
    GenerateShrinkwrapTask.prototype._dangerouslyDeletePath = function (folderPath) {
        try {
            rimraf.sync(folderPath);
        }
        catch (e) {
            throw new Error("" + e.message + os.EOL + "Often this is caused by a file lock from a process\n       such as your text editor, command prompt, or \"gulp serve\"");
        }
    };
    return GenerateShrinkwrapTask;
}(GulpTask_1.GulpTask));
exports.GenerateShrinkwrapTask = GenerateShrinkwrapTask;

//# sourceMappingURL=GenerateShrinkwrapTask.js.map
