/// <reference types="es6-promise" />
/// <reference types="node" />
import { GulpProxy } from '../GulpProxy';
import { IExecutable } from '../IExecutable';
import { IBuildConfig } from '../IBuildConfig';
import gulp = require('gulp');
export declare abstract class GulpTask<TASK_CONFIG> implements IExecutable {
    name: string;
    buildConfig: IBuildConfig;
    taskConfig: TASK_CONFIG;
    cleanMatch: string[];
    /**
     * Shallow merges config settings into the task config.
     */
    setConfig(taskConfig: TASK_CONFIG): void;
    /**
     * Deep merges config settings into task config.
     */
    mergeConfig(taskConfig: TASK_CONFIG): void;
    /**
     * Replaces task config settings with new settings.
     */
    replaceConfig(taskConfig: TASK_CONFIG): void;
    isEnabled(buildConfig: IBuildConfig): boolean;
    abstract executeTask(gulp: gulp.Gulp | GulpProxy, completeCallback?: (result?: Object) => void): Promise<Object> | NodeJS.ReadWriteStream | void;
    log(message: string): void;
    logVerbose(message: string): void;
    logWarning(message: string): void;
    logError(message: string): void;
    fileError(filePath: string, line: number, column: number, errorCode: string, message: string): void;
    fileWarning(filePath: string, line: number, column: number, errorCode: string, message: string): void;
    getCleanMatch(buildConfig: IBuildConfig, taskConfig?: TASK_CONFIG): string[];
    execute(config: IBuildConfig): Promise<void>;
    resolvePath(localPath: string): string;
    fileExists(localPath: string): boolean;
    copyFile(localSourcePath: string, localDestPath?: string): void;
    readJSONSync(localPath: string): Object;
}
