import { ITodoItem, IDataProvider } from './types/index';
/**
 * DataProvider the class used to maintain the client side data.
 *
 * It interact with data provider to sync data items with sharepoint list.
 * It also maintain the loading state to tell React Component once the action is
 * started of finished.
 */
export default class DataProvider implements IDataProvider {
    /**
     * The counter used to identify the latest server call.
     * If the current call is not the latest one, we don't want to update the list
     * to mess up with the out of date data with data in this store.
     */
    private _items;
    private _isLoading;
    private _listeners;
    /**
     * The items store in the local. It only contains the data recently fetched from server.
     */
    items: Array<ITodoItem>;
    /**
     * Whether there is unfinished server request currently.
     */
    isLoading: boolean;
    constructor();
    /**
     * Create a new item and add it to the list through data provider.
     */
    createItem(title: string): Promise<ITodoItem[]>;
    /**
     * Delete a item from the list through data provider.
     */
    deleteItem(delItem: ITodoItem): Promise<ITodoItem[]>;
    /**
     * Toggle the complete state of an item by.
     *
     * Will call updateItem function to update complete state of this item.
     */
    toggleComplete(item: ITodoItem): Promise<ITodoItem[]>;
    /**
     * Add listener to the provider.
     *
     * Once the store has a change of state and emit that change, the listeners will be called.
     */
    addListener(listener: () => void): void;
    /**
     * Remove the registered listener.
     *
     * You must remove the listener registered by addListener method when you are not going to use it anymore.
     */
    removeListener(listener: () => void): void;
    /**
     * Emit the changes in the store to all listeners.
     */
    private _emitChange();
    private _generateGuid();
}
