"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var React = require('react');
var Spinner_1 = require('office-ui-fabric-react/lib/Spinner');
var TodoForm_1 = require('./TodoForm');
var TodoTabs_1 = require('./TodoTabs');
var Todo_module_scss_1 = require('./Todo.module.scss');
var strings_1 = require('./../strings');
/**
 * Todo component is the top level react component of this web part.
 * It uses fabric-react component <Spinner>
 *
 * Link of Spinner: https://fabricreact.azurewebsites.net/fabric-react/master/#/examples/spinner
 */
var Todo = (function (_super) {
    __extends(Todo, _super);
    function Todo(props) {
        _super.call(this, props);
        this._onProviderChange = this._onProviderChange.bind(this);
        this.state = {
            items: this.props.dataProvider.items
        };
    }
    Todo.prototype.componentDidMount = function () {
        this.props.dataProvider.addListener(this._onProviderChange);
    };
    Todo.prototype.componentWillUnmount = function () {
        this.props.dataProvider.removeListener(this._onProviderChange);
    };
    Todo.prototype.render = function () {
        return (React.createElement("div", {className: Todo_module_scss_1.default.todo}, 
            React.createElement("div", {className: Todo_module_scss_1.default.topRow}, 
                React.createElement("h2", {className: Todo_module_scss_1.default.todoHeading}, strings_1.default.todoListTitle), 
                this._renderWorkingOnItSpinner()), 
            React.createElement(TodoForm_1.default, {onSubmit: this.props.dataProvider.createItem}), 
            React.createElement(TodoTabs_1.default, {items: this.state.items, onToggleComplete: this.props.dataProvider.toggleComplete, onDeleteItem: this.props.dataProvider.deleteItem}), 
            this._renderFetchingTasksSpinner()));
    };
    Todo.prototype._renderWorkingOnItSpinner = function () {
        return this.props.dataProvider.isLoading && this.state.items.length > 0
            ? React.createElement("div", {className: Todo_module_scss_1.default.workingOnItSpinner}, 
                React.createElement(Spinner_1.Spinner, {type: Spinner_1.SpinnerType.normal})
            )
            : undefined;
    };
    Todo.prototype._renderFetchingTasksSpinner = function () {
        return this.props.dataProvider.isLoading && this.state.items.length === 0
            ? React.createElement("div", {className: Todo_module_scss_1.default.fetchingTasksSpinner}, 
                React.createElement(Spinner_1.Spinner, {type: Spinner_1.SpinnerType.large, label: strings_1.default.fetchingTasksLabel})
            )
            : undefined;
    };
    Todo.prototype._onProviderChange = function () {
        this.setState({
            items: this.props.dataProvider.items
        });
    };
    return Todo;
}(React.Component));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Todo;

//# sourceMappingURL=Todo.js.map
