"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var React = require('react');
var Button_1 = require('office-ui-fabric-react/lib/Button');
var Checkbox_1 = require('office-ui-fabric-react/lib/Checkbox');
var FocusZone_1 = require('office-ui-fabric-react/lib/FocusZone');
var css_1 = require('office-ui-fabric-react/lib/utilities/css');
var Todo_module_scss_1 = require('./Todo.module.scss');
var strings_1 = require('./../strings');
/**
 * TodoItem component using fabric-react component <FocusZone> <Checkbox> <Button> <DocumentCardActivity>.
 *
 * Link of FocusZone: https://fabricreact.azurewebsites.net/fabric-react/master/#examples/focuszone
 * Link of Checkbox: https://fabricreact.azurewebsites.net/fabric-react/master/#/examples/checkbox
 * Link of Button: https://fabricreact.azurewebsites.net/fabric-react/master/#/examples/button
 * Link of DocumentCardActivity: https://fabricreact.azurewebsites.net/fabric-react/master/#/examples/documentcard
 */
var TodoItem = (function (_super) {
    __extends(TodoItem, _super);
    function TodoItem(props) {
        _super.call(this, props);
        this._onCheckboxChange = this._onCheckboxChange.bind(this);
        this._onDelete = this._onDelete.bind(this);
    }
    TodoItem.prototype.componentWillUnmount = function () {
        window.clearTimeout(this._animationTimeoutId);
    };
    TodoItem.prototype.render = function () {
        var _this = this;
        var className = css_1.css(Todo_module_scss_1.default.todoItem, this.props.item.isComplete === true ? Todo_module_scss_1.default.isCompleted : '', 'ms-Grid', 'ms-u-slideDownIn20');
        return (React.createElement("div", {role: 'row', ref: function (ref) { return _this._rowItem = ref; }, className: className, "aria-label": this._ariaLabel, "data-is-focusable": true}, 
            React.createElement(FocusZone_1.FocusZone, {direction: FocusZone_1.FocusZoneDirection.horizontal}, 
                React.createElement("div", {className: css_1.css(Todo_module_scss_1.default.itemTaskRow, 'ms-Grid-row')}, 
                    React.createElement(Checkbox_1.Checkbox, {label: this.props.item.title, onChange: this._onCheckboxChange, checked: this.props.item.isComplete === true}), 
                    React.createElement(Button_1.Button, {className: Todo_module_scss_1.default.deleteButton, buttonType: Button_1.ButtonType.icon, icon: 'x', onClick: this._onDelete, title: strings_1.default.deleteItemTitle, ariaLabel: strings_1.default.deleteItemAriaLabel}))
            )
        ));
    };
    Object.defineProperty(TodoItem.prototype, "_ariaLabel", {
        get: function () {
            var completeState = this.props.item.isComplete === true
                ? strings_1.default.todoItemAriaLabelCheckedState
                : strings_1.default.todoItemAriaLabelUncheckedState;
            var titleString = strings_1.default.todoItemAriaLabelTitle + this.props.item.title;
            return completeState + " " + titleString;
        },
        enumerable: true,
        configurable: true
    });
    TodoItem.prototype._onCheckboxChange = function (ev, isChecked) {
        this._handleWithAnimation(this.props.onToggleComplete, 'ms-u-slideUpOut20');
    };
    TodoItem.prototype._onDelete = function (event) {
        this._handleWithAnimation(this.props.onDeleteItem, 'ms-u-slideUpOut20');
    };
    TodoItem.prototype._handleWithAnimation = function (callback, animationClass) {
        var _this = this;
        this._rowItem.classList.add(animationClass);
        window.clearTimeout(this._animationTimeoutId);
        this._animationTimeoutId = window.setTimeout(function () {
            _this._rowItem.classList.add(Todo_module_scss_1.default.isHidden);
            callback(_this.props.item);
        }, TodoItem.ANIMATION_TIMEOUT);
    };
    TodoItem.ANIMATION_TIMEOUT = 200;
    return TodoItem;
}(React.Component));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = TodoItem;

//# sourceMappingURL=TodoItem.js.map
