"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var React = require('react');
var Pivot_1 = require('office-ui-fabric-react/lib/Pivot');
var FocusZone_1 = require('office-ui-fabric-react/lib/FocusZone');
var List_1 = require('office-ui-fabric-react/lib/List');
var KeyCodes_1 = require('office-ui-fabric-react/lib/utilities/KeyCodes');
var TodoItem_1 = require('./TodoItem');
var Todo_module_scss_1 = require('./Todo.module.scss');
var strings_1 = require('./../strings');
/**
 * The TodoTabs component using fabric-react component <Pivot> <List> <FocusZone>.
 *
 * Link of <Pivot>: https://fabricreact.azurewebsites.net/fabric-react/master/#/examples/pivot
 * Link of <List>: https://fabricreact.azurewebsites.net/fabric-react/master/#/examples/list
 * Link of <FocusZone>: https://fabricreact.azurewebsites.net/fabric-react/master/#examples/focuszone
 */
var TodoTabs = (function (_super) {
    __extends(TodoTabs, _super);
    function TodoTabs() {
        _super.apply(this, arguments);
    }
    TodoTabs.prototype.render = function () {
        var pivotArray = [];
        var activeTasks = this.props.items.filter(function (task) { return task.isComplete === false; });
        var completedTasks = this.props.items.filter(function (task) { return task.isComplete === true; });
        var allTasks = activeTasks.concat(completedTasks);
        this._onRenderTodoItem = this._onRenderTodoItem.bind(this);
        if (activeTasks.length > 0) {
            pivotArray.push(this._renderPivotItemList(activeTasks, strings_1.default.todoListTabNameActive));
        }
        if (completedTasks.length > 0) {
            pivotArray.push(this._renderPivotItemList(completedTasks, strings_1.default.todoListTabNameCompleted));
        }
        if (allTasks.length > 0) {
            pivotArray.push(this._renderPivotItemList(allTasks, strings_1.default.todoListTabNameAllTasks));
        }
        return pivotArray.length > 0
            ? (React.createElement("div", {className: Todo_module_scss_1.default.todoPivot}, 
                React.createElement(Pivot_1.Pivot, {linkSize: Pivot_1.PivotLinkSize.large}, pivotArray)
            ))
            : null; // tslint:disable-line:no-null-keyword
    };
    TodoTabs.prototype._renderPivotItemList = function (tasks, tabName) {
        // @todo #219004 make isInnerZoneKeystroke be rtl safe.
        return (React.createElement(Pivot_1.PivotItem, {linkText: tabName + " (" + tasks.length + ")"}, 
            React.createElement(FocusZone_1.FocusZone, {direction: FocusZone_1.FocusZoneDirection.vertical, isInnerZoneKeystroke: function (ev) { return ev.which === KeyCodes_1.KeyCodes.right; }}, 
                React.createElement(List_1.List, {className: Todo_module_scss_1.default.todoList, items: tasks, onRenderCell: this._onRenderTodoItem})
            )
        ));
    };
    TodoTabs.prototype._onRenderTodoItem = function (item) {
        return (React.createElement(TodoItem_1.default, {key: item.id, item: item, onToggleComplete: this.props.onToggleComplete, onDeleteItem: this.props.onDeleteItem}));
    };
    return TodoTabs;
}(React.Component));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = TodoTabs;

//# sourceMappingURL=TodoTabs.js.map
