import * as React from 'react';
import { Pane } from './Pane';
export interface IPaneProps extends React.Props<Pane> {
    /**
     * Whether the pane is displayed.
     * @default false
     */
    hidden?: boolean;
    /**
     * Mode of the pane.
     * @default PaneMode.push
     */
    paneMode?: PaneMode;
    /**
    * Has the close button visible.
    * @default true
    */
    closeButtonVisible?: boolean;
    /**
     * Additional styling options.
     */
    className?: string;
    /**
     * Aria label on close button
     */
    closeButtonAriaLabel?: string;
    /**
     * Header text for the Pane.
     * @default ""
     */
    headerText?: string;
    /**
     * Optional parameter to provider the class name for header text
     */
    headerClassName?: string;
    /**
     * Type of the pane.
     * @default PaneType.small
     */
    type?: PaneType;
    /**
     * Event handler for when the pane is closed.
     */
    onDismiss?: () => void;
}
export declare enum PaneType {
    /**
     * Renders the pane in 'small' mode, anchored to the far side (right in LTR mode).
     */
    small = 0,
    /**
     * Renders the pane in 'medium' mode, anchored to the far side (right in LTR mode).
     */
    medium = 1,
}
export declare enum PaneMode {
    /**
     * Pane reflows the main content and shrinks the viewable area.
     */
    push = 0,
    /**
     * Pane leaves the content unchanged and only resizes the viewable area.
     */
    overlay = 1,
}
