"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
/* tslint:disable:no-unused-variable */
var React = require('react');
/* tslint:enable:no-unused-variable */
var BaseComponent_1 = require('../../common/BaseComponent');
var Pane_Props_1 = require('./Pane.Props');
var index_1 = require('../Popup/index');
var css_1 = require('../../utilities/css');
var object_1 = require('../../utilities/object');
var rtl_1 = require('../../utilities/rtl');
var PaneContent_1 = require('./PaneContent');
var WrappedContent_1 = require('./WrappedContent');
require('./Pane.scss');
var Pane = (function (_super) {
    __extends(Pane, _super);
    function Pane(props) {
        _super.call(this, props);
        this._onClose = this._onClose.bind(this);
        this._onPaneRef = this._onPaneRef.bind(this);
        this._id = object_1.getId('Pane');
        this.state = {
            hidden: props.hidden,
            isAnimatingOpen: !!props.hidden,
            isAnimatingClose: false
        };
    }
    Pane.prototype.componentDidMount = function () {
        var _this = this;
        // Set original content width for overlay mode
        this._initialContentWidth = this._paneControlContainer.getBoundingClientRect().width;
        if (!this.state.hidden) {
            this._async.setTimeout(function () {
                _this.setState({
                    isAnimatingOpen: false
                });
            }, 2000);
        }
        this._events.on(window, 'resize', this._updateRenderedItems);
    };
    Pane.prototype.componentWillReceiveProps = function (newProps) {
        if (newProps.hidden !== this.state.hidden) {
            this.setState({
                hidden: newProps.hidden,
                isAnimatingOpen: newProps.hidden ? false : true,
                isAnimatingClose: newProps.hidden ? true : false
            });
        }
    };
    Pane.prototype.componentDidUpdate = function () {
        if (!this.state.hidden) {
            this._contentContainer.style.width = '';
        }
        else {
            if (this.props.paneMode === Pane_Props_1.PaneMode.overlay) {
                // Use original content width for overlay mode
                this._contentContainer.style.width = this._initialContentWidth + 'px';
            }
            else if (this.props.paneMode === Pane_Props_1.PaneMode.push) {
                // Viewport content width for push mode
                this._contentContainer.style.width = this._getContainerWidth() + 'px';
            }
        }
    };
    Pane.prototype.componentWillUnmount = function () {
        this._events.dispose();
    };
    Pane.prototype.render = function () {
        var _a = this.props, _b = _a.className, className = _b === void 0 ? '' : _b, type = _a.type, closeButtonVisible = _a.closeButtonVisible, headerText = _a.headerText, closeButtonAriaLabel = _a.closeButtonAriaLabel, _c = _a.headerClassName, headerClassName = _c === void 0 ? '' : _c;
        var _d = this.state, hidden = _d.hidden, isAnimatingOpen = _d.isAnimatingOpen, isAnimatingClose = _d.isAnimatingClose;
        var isRTL = rtl_1.getRTL();
        var isOnRightSide = !isRTL;
        var headerTextId = this._id + '-headerText';
        var pendingCommandBarContent = '';
        var header = headerText ? (React.createElement("p", {className: css_1.css('ms-Pane-headerText', headerClassName), id: headerTextId}, headerText)) : null;
        var closeButton = closeButtonVisible ?
            (React.createElement("button", {className: 'ms-Pane-closeButton ms-PaneAction-close', onClick: this._onClose, "aria-label": closeButtonAriaLabel, "data-is-visible": true}, 
                React.createElement("i", {className: 'ms-Pane-closeIcon ms-Icon ms-Icon--Cancel'})
            )) : null;
        var groupings = this._groupChildren();
        return (React.createElement("div", {className: css_1.css('ms-Pane', {
            'ms-Pane--left': !isOnRightSide,
            'ms-Pane--right': isOnRightSide
        }), ref: this._resolveRef('_paneControlContainer')}, 
            React.createElement("div", {className: 'main-content', ref: this._resolveRef('_mainContent')}, 
                React.createElement("div", {className: 'content-container', ref: this._resolveRef('_contentContainer')}, groupings.wrappedContents)
            ), 
            React.createElement(index_1.Popup, {className: 'ms-Pane-popup', role: 'dialog', ariaLabelledBy: headerText ? headerTextId : undefined, onDismiss: this.props.onDismiss}, 
                React.createElement("div", {ref: this._onPaneRef, className: css_1.css('ms-Pane', className, {
                    'ms-Pane--openLeft': !isOnRightSide,
                    'ms-Pane--openRight': isOnRightSide,
                    'open': !hidden,
                    'ms-Pane-animateIn': isAnimatingOpen,
                    'ms-Pane-animateOut': isAnimatingClose,
                    'ms-Pane--sm': type === Pane_Props_1.PaneType.small,
                    'ms-Pane--md': type === Pane_Props_1.PaneType.medium
                })}, 
                    React.createElement("div", {className: 'ms-Pane-main'}, 
                        React.createElement("div", {className: 'ms-Pane-commands', "data-is-visible": true}, 
                            pendingCommandBarContent, 
                            closeButton), 
                        React.createElement("div", {className: 'ms-Pane-contentInner'}, 
                            header, 
                            React.createElement("div", {className: 'ms-Pane-content'}, groupings.paneContents)))
                )
            )));
    };
    Pane.prototype.dismiss = function () {
        if (!this.state.hidden) {
            this.setState({
                isAnimatingOpen: false,
                isAnimatingClose: true
            });
        }
    };
    Pane.prototype._getContainerWidth = function () {
        return this._mainContent.getBoundingClientRect().width;
    };
    Pane.prototype._groupChildren = function () {
        var groupings = {
            wrappedContents: [],
            paneContents: []
        };
        React.Children.map(this.props.children, function (child) {
            if (typeof child === 'object' && child !== null) {
                if (child.type === PaneContent_1.PaneContent) {
                    groupings.paneContents.push(child);
                }
                else if (child.type === WrappedContent_1.WrappedContent) {
                    groupings.wrappedContents.push(child);
                }
            }
        });
        return groupings;
    };
    Pane.prototype._updateRenderedItems = function () {
        // Set original content width for overlay mode
        this._initialContentWidth = this._paneControlContainer.getBoundingClientRect().width;
        this.setState(this.state);
    };
    Pane.prototype._onAnimationEnd = function (ev) {
        if (ev.animationName.indexOf('In') > -1) {
            this.setState({
                hidden: false,
                isAnimatingOpen: false
            });
        }
        if (ev.animationName.indexOf('Out') > -1) {
            this.setState({
                hidden: true,
                isAnimatingClose: false
            });
            if (this.props.onDismiss) {
                this.props.onDismiss();
            }
        }
    };
    Pane.prototype._onClose = function () {
        this.dismiss();
    };
    Pane.prototype._onPaneRef = function (ref) {
        if (ref) {
            this._events.on(ref, 'animationend', this._onAnimationEnd);
        }
        else {
            this._events.off();
        }
    };
    Pane.defaultProps = {
        hidden: true,
        closeButtonVisible: true,
        type: Pane_Props_1.PaneType.small
    };
    return Pane;
}(BaseComponent_1.BaseComponent));
exports.Pane = Pane;

//# sourceMappingURL=Pane.js.map
